package jaredbgreat.dldungeons.pieces.entrances;

import jaredbgreat.dldungeons.builder.Builder;
import jaredbgreat.dldungeons.planner.Dungeon;
import net.minecraft.block.Block;
import net.minecraft.world.World;

public class TopRoom extends AbstractEntrance {

	
	public TopRoom(int x, int y, int z) {
		super(x, y, z);
		
	}

	
	@Override
	public void build(Dungeon dungeon, World world) {
		int wx = x + (dungeon.map.chunkX * 16) - (dungeon.map.room.length / 2) + 8;
		int wz = z + (dungeon.map.chunkZ * 16) - (dungeon.map.room.length / 2) + 8;
		int bottom = dungeon.map.floorY[x][z];
		int top = world.getHeightValue(wx, wz);
		while(!Builder.isGroundBlock(world, wx, top, wz)) top--;
		int xdim = dungeon.random.nextInt(7) + 6;
		int zdim = dungeon.random.nextInt(7) + 6;
		int ymod = (xdim <= zdim) ? (int) Math.sqrt(xdim) : (int) Math.sqrt(zdim);
		int ydim = dungeon.random.nextInt((dungeon.verticle.value / 2) + (ymod / 2) + 2) + 2;
		
		int xmin = wx - (xdim / 2);
		int xmax = wx + (xdim / 2);
		int zmin = wz - (zdim / 2);
		int zmax = wz + (zdim / 2);
		
		int below;
		if(dungeon.random.nextBoolean() || dungeon.degeneracy.use(dungeon.random)) {
			for(int i = xmin + 1; i < xmax; i++) 
				for(int j = zmin + 1; j < zmax; j++) {
					Builder.placeBlock(world, i, top, j, dungeon.floorBlock);
					below = top - 1;
					while(!Builder.isGroundBlock(world, i, below, j)) {
						Builder.placeBlock(world, i, below, j, dungeon.floorBlock);
						below--;
					}
				}
			for(int i = xmin; i <= xmax; i++) {
				for(int j = top + ydim - dungeon.random.nextInt(3); j >= top; j--) { 
					Builder.placeBlock(world, i, j, zmax, dungeon.wallBlock1);
					Builder.placeBlock(world, i, j, zmin, dungeon.wallBlock1);
				}
				if(dungeon.random.nextInt(8) == 0)
					for(int j = top + 2; j > top; j--) {
						Builder.deleteBlock(world, i, j, zmax);
						Builder.deleteBlock(world, i, j, zmin);
					}
				below = top - 1;
				while(!Builder.isGroundBlock(world, i, below, zmax)) {
					Builder.placeBlock(world, i, below, zmax, dungeon.wallBlock1);
					below--;
				}
				below = top - 1;
				while(!Builder.isGroundBlock(world, i, below, zmin)) {
					Builder.placeBlock(world, i, below, zmin, dungeon.floorBlock);
					below--;
				}
			}
			for(int i = zmin; i <= zmax; i++) {
				for(int j = top + ydim - dungeon.random.nextInt(3); j >= top; j--) { 
					Builder.placeBlock(world, xmin, j, i, dungeon.wallBlock1);
					Builder.placeBlock(world, xmax, j, i, dungeon.wallBlock1);
				}
				if(dungeon.random.nextInt(8) == 0)
					for(int j = top + 2; j > top; j--) {
						Builder.deleteBlock(world, xmax, j, i);
						Builder.deleteBlock(world, xmin, j, i);
					}
				below = top - 1;
				while(!Builder.isGroundBlock(world, xmax, below, i)) {
					Builder.placeBlock(world, xmax, below, i, dungeon.wallBlock1);
					below--;
				}
				below = top - 1;
				while(!Builder.isGroundBlock(world, xmin, below, i)) {
					Builder.placeBlock(world, xmin, below, i, dungeon.floorBlock);
					below--;
				}
			}
		} else {
			for(int i = xmin + 1; i < xmax; i++) 
				for(int j = zmin + 1; j < zmax; j++) {
					Builder.placeBlock(world, i, top, j, dungeon.floorBlock);
					below = top;
					for(int k = top + (ydim * 2); k >= top + 1; k--) 
						Builder.deleteBlock(world, i, k, j);
					while(!Builder.isGroundBlock(world, i, below, j)) {
						Builder.placeBlock(world, i, below, j, dungeon.floorBlock);
					}
				}
			for(int i = xmin; i <= xmax; i++) {
				for(int j = top + ydim; j >= top; j--) { 
					Builder.placeBlock(world, i, j, zmax, dungeon.wallBlock1);
					Builder.placeBlock(world, i, j, zmin, dungeon.wallBlock1);
				}
				if(dungeon.random.nextInt(8) == 0)
					for(int j = top + 2; j > top; j--) {
						Builder.deleteBlock(world, i, j, zmax);
						Builder.deleteBlock(world, i, j, zmin);
					}
				below = top - 1;
				while(!Builder.isGroundBlock(world, i, below, zmax)) {
					Builder.placeBlock(world, i, below, zmax, dungeon.wallBlock1);
					below--;
				}
				below = top - 1;
				while(!Builder.isGroundBlock(world, i, below, zmin)) {
					Builder.placeBlock(world, i, below, zmin, dungeon.floorBlock);
					below--;
				}
			}
			for(int i = zmin; i <= zmax; i++) {
				for(int j = top + ydim; j >= top; j--) { 
					Builder.placeBlock(world, xmin, j, i, dungeon.wallBlock1);
					Builder.placeBlock(world, xmax, j, i, dungeon.wallBlock1);
				}
				if(dungeon.random.nextInt(8) == 0)
					for(int j = top + 2; j > top; j--) {
						Builder.deleteBlock(world, xmax, j, i);
						Builder.deleteBlock(world, xmin, j, i);
					}
				below = top - 1;
				while(!Builder.isGroundBlock(world, xmax, below, i)) {
					Builder.placeBlock(world, xmax, below, i, dungeon.wallBlock1);
					below--;
				}
				below = top - 1;
				while(!Builder.isGroundBlock(world, xmin, below, i)) {
					Builder.placeBlock(world, xmin, below, i, dungeon.floorBlock);
					below--;
				}
			}
			for(int i = xmin; i <= xmax; i++) 
				for(int j = zmin; j <= zmax; j++) {
					Builder.placeBlock(world, i, top + ydim + 1, j, dungeon.cielingBlock);
				}
			}
		
			if(dungeon.random.nextBoolean()) {
				top += ydim;
				int side = dungeon.random.nextInt(4);
				switch (side) {
					case 0:
						for(int i = bottom; i <= top; i++) {
							Builder.placeBlock(world, wx, i, wz, dungeon.wallBlock1);
							Builder.placeBlock(world, wx + 1, i, wz, Block.ladder.blockID, 5, 3);
						}
						break;
					case 1:
						for(int i = bottom; i <= top; i++) {
							Builder.placeBlock(world, wx, i, wz, dungeon.wallBlock1);
							Builder.placeBlock(world, wx, i, wz + 1, Block.ladder.blockID, 3, 3);
						}
						break;
					case 2:
						for(int i = bottom; i <= top; i++) {
							Builder.placeBlock(world, wx, i, wz, dungeon.wallBlock1);
							Builder.placeBlock(world, wx - 1, i, wz, Block.ladder.blockID, 4, 3);
						}
						break;
					case 3:
						for(int i = bottom; i <= top; i++) {
							Builder.placeBlock(world, wx, i, wz, dungeon.wallBlock1);
							Builder.placeBlock(world, wx, i, wz - 1, Block.ladder.blockID, 2, 3);
						}
						break;
				}				
			} else {
				top++;
				int side = dungeon.random.nextInt(4);
				for(int i = bottom; i < top; i++) {
					int sx, sz;
					Builder.placeBlock(world, wx, i, wz, dungeon.wallBlock1);
					switch (side) {
					case 0:
						Builder.placeBlock(world, wx+1, i, wz,   Block.stoneSingleSlab.blockID, 0, 3);
						Builder.placeBlock(world, wx+1, i, wz+1, Block.stoneSingleSlab.blockID, 8, 3);
						// Empty space
						Builder.deleteBlock(world, wx,    i, wz+1);
						Builder.deleteBlock(world, wx-1,  i, wz+1);
						Builder.deleteBlock(world, wx-1,  i,   wz);
						Builder.deleteBlock(world, wx-1,  i, wz-1);
						Builder.deleteBlock(world, wx,    i, wz-1);
						Builder.deleteBlock(world, wx+1,  i, wz-1);
						break;
					case 1:
						Builder.placeBlock(world, wx,   i, wz+1, Block.stoneSingleSlab.blockID, 0, 3);
						Builder.placeBlock(world, wx-1, i, wz+1, Block.stoneSingleSlab.blockID, 8, 3);
						// Empty space
						Builder.deleteBlock(world, wx+1, i,   wz);
						Builder.deleteBlock(world, wx+1, i, wz+1);
						Builder.deleteBlock(world, wx-1, i,   wz);
						Builder.deleteBlock(world, wx-1, i, wz-1);
						Builder.deleteBlock(world, wx,   i, wz-1);
						Builder.deleteBlock(world, wx+1, i, wz-1);
						break;
					case 2:
						Builder.placeBlock(world, wx-1, i, wz,   Block.stoneSingleSlab.blockID, 0, 3);
						Builder.placeBlock(world, wx-1, i, wz-1, Block.stoneSingleSlab.blockID, 8, 3);
						// Empty space
						Builder.deleteBlock(world, wx+1, i,   wz);
						Builder.deleteBlock(world, wx+1, i, wz+1);
						Builder.deleteBlock(world, wx,   i, wz+1);
						Builder.deleteBlock(world, wx-1, i, wz+1);
						Builder.deleteBlock(world, wx,   i, wz-1);
						Builder.deleteBlock(world, wx+1, i, wz-1);
						break;
					case 3:
						Builder.placeBlock(world, wx,   i, wz-1, Block.stoneSingleSlab.blockID, 0, 3);
						Builder.placeBlock(world, wx+1, i, wz-1, Block.stoneSingleSlab.blockID, 8, 3);
						// Empty space
						Builder.deleteBlock(world, wx+1, i,   wz);
						Builder.deleteBlock(world, wx+1, i, wz+1);
						Builder.deleteBlock(world, wx,   i, wz+1);
						Builder.deleteBlock(world, wx-1, i, wz+1);
						Builder.deleteBlock(world, wx-1, i,   wz);
						Builder.deleteBlock(world, wx-1, i, wz-1);
						break;
					}
					side = (side + 1) % 4; 
				}
			}
		}
	
}
