package jaredbgreat.dldungeons.themes;


/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import jaredbgreat.dldungeons.pieces.chests.LootItem;
import jaredbgreat.dldungeons.pieces.chests.LootList;
import jaredbgreat.dldungeons.pieces.chests.TreasureChest;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.StringTokenizer;

import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraftforge.common.BiomeDictionary.Type;
import cpw.mods.fml.common.registry.GameRegistry;


public class ThemeReader {
	
	
	private static File themesDir;
	private static ArrayList<File> files;
	
	
	public static void setThemesDir(File dir) {
		System.out.println("[DLDUNGEONS] themesdir is " + dir);
		themesDir = dir;
	}
	
	
	private static int findFiles() {
		int num = 0;
		files = new ArrayList<File>();
		String[] fileNames = themesDir.list();
		if(fileNames.length < 1) {
			return 0;
		}
		for(String name : fileNames) {
			if(name.length() >= 5) {
				if(name.equals("template.cfg") || name.equals("chests.cfg")) continue;
				else if(name.substring(name.length() - 4).equals(".cfg")) {
					files.add(new File(name));
					num++;
				}
			}
		}
		return num;
	}

	
	public static void readThemes() {
		Theme.addDefaultTheme();
		int num = findFiles();
		System.out.println("[DLDUNGEONS] Found " + num + " themes.");
		for(File file : files) readTheme(file);
		TreasureChest.initSlots();
		openLoot();		
	}
	
	
	public static void openLoot() {
		BufferedReader instream = null;
		File chests = new File(themesDir.toString() + File.separator + "chests.cfg");
		if(chests.exists()) try {
			instream = new BufferedReader(new 
					FileReader(chests.toString()));
			readLoot(instream);
			if(instream != null) instream.close();
		} catch (IOException e) {
			e.printStackTrace();
		} else {
			System.out.println("[DLDUNGEONS] File chests.cfg is missing; will fallabck on default loot");
			LootList.addDefaultLoot();
		}
	}
	
	
	public static void readLoot(BufferedReader instream) throws IOException {
		System.out.println("[DLDUNGEONS] Loading chest loot file (chests.cfg)");
		
		StringTokenizer tokens = null;
		String line = null;
		String token;
		int itemid;
		Item item;
		Block block;
		
		String type;
		int level;
		String modid;
		String name;
		LootItem loot;
		int min;
		int max;
		
		while((line = instream.readLine()) != null) {
			if(line.length() < 2) continue;
			if(line.charAt(0) == '#') continue;
			//System.out.println(line);
			tokens = new StringTokenizer(line, " ,.;:\t\n\r\f=");
			if(!tokens.hasMoreTokens()) continue;
			type = tokens.nextToken().toLowerCase();
			//System.out.println("type = " + type);
			if(!tokens.hasMoreTokens()) continue;
			level = intParser(tokens);
			//System.out.println("level = " + level);
			if(!tokens.hasMoreTokens() || (level == 0)) continue;
//			modid = tokens.nextToken();
//			if(modid.toLowerCase().equals("item") || modid.toLowerCase().equals("minecraft"))
//				modid = "Minecraft";
//			//System.out.println("modid = " + modid);
//			if(!tokens.hasMoreTokens()) continue;
//			name = tokens.nextToken();
//			//System.out.println("name = " + name);
			itemid = intParser(tokens);
			if(!tokens.hasMoreTokens()) continue;
			min = intParser(tokens);
			//System.out.println("min = " + min);
			if(!tokens.hasMoreTokens() || (min == 0)) continue;
			max = intParser(tokens);
			if(max == 0) continue;
//			if(tokens.hasMoreTokens()) 
//				if(booleanParser(false, tokens)) {
//					//System.out.println("Is a block");
//					block = GameRegistry.findBlock(modid, name);
//					loot = new LootItem(block, min, max);
//					System.out.println("[DLDUNGEONS] Adding chest loot: " + block);
//					if(block != null && loot != null) {
//						LootList.addItem(loot, type, level);
//					}
//					System.out.println("[DLDUNGEONS] Added chest loot: " + type + level + ", " 
//							+ modid + "." + name + ", min: " + min + ", max: " + max);
//					continue;
//				} 
			//System.out.println("Is NOT a block");
			item = Item.itemsList[itemid];
			loot = new LootItem(item, min, max);
//			System.out.println("[DLDUNGEONS] Adding chest loot: " + item);
			if(item != null && loot != null) LootList.addItem(loot, type, level);
//			System.out.println("[DLDUNGEONS] Added chest loot: " + type + level + ", " 
//					+ modid + "." + name + ", min: " + min + ", max: " + max);
		}
	}			
		
	
	private static void readTheme(File file) {
		BufferedReader instream = null;
		try {
			instream = new BufferedReader(new 
					FileReader(themesDir.toString() + File.separator + file.toString()));
			System.out.println("[DLDUNGEONS] Loading theme file " + file.toString());
			parseTheme(instream, file.toString());
			if(instream != null) instream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	private static void parseTheme(BufferedReader instream, String name) throws IOException {
		Theme theme = new Theme();
		theme.name = name;
		StringTokenizer tokens = null;
		String line = null;
		String token;
		while((line = instream.readLine()) != null) {
			if(line.length() < 2) continue;
			if(line.charAt(0) == '#') continue;
			tokens = new StringTokenizer(line, " ,;:\t\n\r\f=");
			if(!tokens.hasMoreTokens()) continue;
			token = tokens.nextToken().toLowerCase();
			if(token.equals("miny")) {
				theme.minY = intParser(theme.minY, tokens);
				continue;
			} if(token.equals("maxy")) {
				theme.maxY = intParser(theme.maxY, tokens);
				continue;
			} if(token.equals("buildfoundation")) {
				theme.buildFoundation = booleanParser(theme.buildFoundation, tokens);
				continue;
			} if(token.equals("sizes")) {
				theme.sizes = sizeParser(theme.sizes, tokens);
				continue;
			} if(token.equals("outside")) {
				theme.outside = elementParser(theme.outside, tokens);
				continue;
			} if(token.equals("liquids")) {
				theme.liquids = elementParser(theme.liquids, tokens);
				continue;
			} if(token.equals("subrooms")) {
				theme.subrooms = elementParser(theme.subrooms, tokens);
				continue;
			} if(token.equals("islands")) {
				theme.islands = elementParser(theme.islands, tokens);
				continue;
			} if(token.equals("pillars")) {
				theme.pillars = elementParser(theme.pillars, tokens);
				continue;
			} if(token.equals("symmetry")) {
				theme.symmetry = elementParser(theme.symmetry, tokens);
				continue;
			} if(token.equals("variability")) {
				theme.variability = elementParser(theme.variability, tokens);
				continue;
			} if(token.equals("degeneracy")) {
				theme.degeneracy = elementParser(theme.degeneracy, tokens);
				continue;
			} if(token.equals("complexity")) {
				theme.complexity = elementParser(theme.complexity, tokens);
				continue;
			} if(token.equals("verticle")) {
				theme.verticle = elementParser(theme.verticle, tokens);
				continue;
			} if(token.equals("entrances")) {
				theme.entrances = elementParser(theme.entrances, tokens);
				continue;
			} if(token.equals("walls")) {
				theme.walls = blockParser(theme.walls, tokens);
				continue;
			} if(token.equals("floors")) {
				theme.floors = blockParser(theme.floors, tokens);
				continue;
			} if(token.equals("ceilings")) {
				theme.ceilings = blockParser(theme.ceilings, tokens);
				continue;
			} if(token.equals("fencing")) {
				theme.fencing = blockParser(theme.fencing, tokens);
				continue;
			} if(token.equals("liquid")) {
				theme.liquid = blockParser(theme.liquid, tokens);
				continue;
			} if(token.equals("pillarblock")) {
				theme.pillarBlock = blockParser(theme.pillarBlock, tokens);
				continue;
			} if(token.equals("commonmobs")) {
				theme.commonMobs = parseMobs(theme.commonMobs, tokens);
				continue;
			} if(token.equals("hardmobs")) {
				theme.hardMobs = parseMobs(theme.hardMobs, tokens);
				continue;
			} if(token.equals("brutemobs")) {
				theme.bruteMobs = parseMobs(theme.bruteMobs, tokens);
				continue;
			} if(token.equals("elitemobs")) {
				theme.eliteMobs = parseMobs(theme.eliteMobs, tokens);
				continue;
			} if(token.equals("biomes")) {
				theme.biomes = biomeParser(tokens);
				continue;
			} if(token.equals("notinbiomes")) {
				theme.notIn = biomeParser(tokens);
				continue;
			}
		}
		theme.fixMobs();
		theme.biomeRegister();
	}
	
	
	private static Element elementParser(Element el, StringTokenizer tokens) {
		boolean valid = false;
		int[] values = new int[]{0, 0, 0, 0, 0, 0};
		String num;
		for(int i = 0; (i < values.length) && tokens.hasMoreTokens(); i++) {
			num = tokens.nextToken();
			values[i] = Integer.parseInt(num);
			if(values[i] < 0) values[i] = 0;
			if(values[i] > 0) valid = true;
		}
		if(valid) return new Element(values[0], values[1], values[2], values[3], values[4], values[5]);
		else return el;
	}
	
	
	private static SizeElement sizeParser(SizeElement el, StringTokenizer tokens) {
		boolean valid = false;
		int[] values = new int[]{0, 0, 0, 0, 0};
		String num;
		for(int i = 0; (i < values.length) && tokens.hasMoreTokens(); i++) {
			num = tokens.nextToken();
			values[i] = Integer.parseInt(num);
			if(values[i] < 0) values[i] = 0;
			if(values[i] > 0) valid = true;
		}
		if(valid) return new SizeElement(values[0], values[1], values[2], values[3], values[4]);
		else return el;
	}
	
	
	private static int intParser(int el, StringTokenizer tokens) {
		boolean valid = false;
		int value = 0;
		String num;
		try {
			if(tokens.hasMoreTokens()) {
				num = tokens.nextToken();
				value = Integer.parseInt(num);
				if((value > 5) && (value < 224)) valid = true;
			}
		} catch(Exception e) {
			System.err.println("[DLDUNGEONS] ThemeReader.intParser(int el, StringTokenizer tokens) tried to read non-number as integer");
			return el;
		}
		if(valid) return value;
		else return el;
	}
	
	
	private static int intParser(StringTokenizer tokens) {
		int value = 0;
		String num;
		try {
			if(tokens.hasMoreTokens()) {
				num = tokens.nextToken();
				value = Integer.parseInt(num);
			}
		} catch(Exception e) {
			System.err.println("[DLDUNGEONS] ThemeReader.intParser(StringTokenizer tokens) tried to read non-number as integer");
			return -1;
		}
		return value;
	}
	
	
	private static boolean booleanParser(boolean el, StringTokenizer tokens) {
		boolean valid = false;
		boolean bool;
		if(tokens.hasMoreTokens()) {
			bool = Boolean.parseBoolean(tokens.nextToken());
		} else return el;
		return bool;
	}
	
	
	private static int[] blockParser(int[] el, StringTokenizer tokens) {
		boolean valid = false;
		ArrayList<String> values = new ArrayList<String>();
		String num;
		int ber;
		while(tokens.hasMoreTokens()) {
			num = tokens.nextToken();
			ber = Integer.parseInt(num);
			if(ber >= 0 && ber < 4096) {
				values.add(num);
				valid = true;
			}
		}
		if(valid) {
			int[] out = new int[values.size()];
			for(int i = 0; i < out.length; i++) out[i] = Integer.parseInt(values.get(i));
			return out;
		}
		else return el;
	}
	
	
	private static ArrayList<String> parseMobs(ArrayList<String> el, StringTokenizer tokens) {
		ArrayList<String> mobs = new ArrayList<String>();
		while(tokens.hasMoreTokens()) {
			String nextMob = tokens.nextToken();
			mobs.add(nextMob);
			System.out.println("[DLDUNGEONS] Adding " + nextMob);
		}
		return mobs;
	}
	
	
	private static EnumSet<Type> biomeParser(StringTokenizer tokens) {
		String name;
		EnumSet<Type> biomes = EnumSet.noneOf(Type.class);
		while(tokens.hasMoreTokens()) {
			name = tokens.nextToken().toUpperCase();
			if(name.equals("FOREST")) biomes.add(Type.FOREST);
			else if(name.equals("PLAINS")) biomes.add(Type.PLAINS);
			else if(name.equals("MOUNTAINS")) biomes.add(Type.MOUNTAIN);
			else if(name.equals("HILLS")) biomes.add(Type.HILLS);
			else if(name.equals("SWAMP")) biomes.add(Type.SWAMP);
			else if(name.equals("WATER")) biomes.add(Type.WATER);
			else if(name.equals("DESERT")) biomes.add(Type.DESERT);
			else if(name.equals("FROZEN")) biomes.add(Type.FROZEN);
			else if(name.equals("JUNGLE")) biomes.add(Type.JUNGLE);
			else if(name.equals("WASTELAND")) biomes.add(Type.WASTELAND);
			else if(name.equals("BEACH")) biomes.add(Type.BEACH);
			else if(name.equals("NETHER")) biomes.add(Type.NETHER);
			else if(name.equals("END")) biomes.add(Type.END);
			else if(name.equals("MUSHROOM")) biomes.add(Type.MUSHROOM);
			else if(name.equals("MAGICAL")) biomes.add(Type.MAGICAL);
		}
		return biomes;
	}
	
		
}
